@@ -1,9 +1,7 @@
-// -*- Mode: C++; indent-tabs-mode: nil; c-basic-offset: 4 -*-
-
 /*
  *  PaperBox - document-tile.cc
  *
- *  Copyright (C) 2007 Marko Anastasov
+ *  Copyright (C) 2007-2008 Marko Anastasov
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -36,29 +34,38 @@ namespace paperbox {
     using boost::shared_ptr;
     using Glib::ustring;
 
+    const unsigned int TITLE_BREAK = 50;
+
     DocumentTile::DocumentTile(Thumbnailer& thumbnailer,
                                const shared_ptr<Document>& doc)
         :
-        Gtk::Util::Tile(doc->get_file_name(), doc->get_author(), false),
+        Gtk::Util::Tile(doc->get_subject(), doc->get_author(), false),
         uri_(doc->get_uri()),
+        doc_(doc),
         tag_hbox_(false, 4),
+        tag_labels_hbox_(false, 4),
+        tag_links_hbox_(false, 4),
         tag_tools_hbox_(false, 4),
         tags_caption_(_("Tags:")),
         image_add_(Gtk::Stock::ADD, Gtk::ICON_SIZE_MENU),
         first_boot_(true)
     {
+        break_title();
+
         connect_thumbnailer_signals(thumbnailer);
 
-        Gtk::VBox&  content_vbox = get_content_vbox();
+        Gtk::VBox& content_vbox = get_content_vbox();
 
         int page_count = doc->get_page_count();
 
+        pages_label_.set_alignment(0.0);
         pages_label_.set_text(
             Glib::Util::uprintf(ngettext("%d page", "%d pages", page_count),
                                 page_count));
 
         extra_hbox_.pack_start(pages_label_, false, false, 0);
 
+        modtime_label_.set_alignment(0.0);
         modtime_label_.set_text(
             Glib::Util::uprintf(_("Modified on %s"),
                                 doc->get_modification_time().c_str()));
@@ -73,11 +80,17 @@ namespace paperbox {
         button_add_tags_.add(image_add_);
         tag_tools_hbox_.pack_start(button_add_tags_, false, false, 0);
 
+        tag_hbox_.pack_start(tag_labels_hbox_, false, false, 0);
+
         // Build tag links from tag string
         vector<ustring> tags = doc->get_tags();
         vector<ustring> removed(0); // none
         refresh_tag_links(tags, removed);
         pack_tag_links();
+
+        refresh_tag_labels(tags, removed);
+        pack_tag_labels();
+
         content_vbox.pack_start(tag_hbox_, false, false, 0);
 
         connect_signals();
@@ -86,6 +99,7 @@ namespace paperbox {
     DocumentTile::~DocumentTile()
     {
         tag_links_.clear();
+        tag_labels_.clear();
     }
 
     ustring
@@ -151,6 +165,31 @@ namespace paperbox {
         return dynamic_cast<Gtk::Widget*>(widget);
     }
 
+    Gtk::Widget*
+    DocumentTile::get_tag_label(const string& tag)
+    {
+        Gtk::Label* widget = 0;
+        tag_label_map::iterator label_iter = tag_labels_.find(tag);
+
+        if (label_iter != tag_labels_.end()) {
+            widget = static_cast<Gtk::Label*>(label_iter->second.get());
+        } else {
+            widget = new Gtk::Label(tag);
+
+            // lighten up the colour a bit
+            GdkColor color;
+            gdk_color_parse ("LightSlateGray", &color);
+            gtk_widget_modify_fg (GTK_WIDGET(widget->gobj()),
+                                  GTK_STATE_NORMAL,
+                                  &color);
+            
+            shared_ptr<Gtk::Widget> shared_widget(widget);
+            tag_labels_[tag] = shared_widget;
+        }
+
+        return dynamic_cast<Gtk::Widget*>(widget);
+    }
+
     void
     DocumentTile::refresh_tag_links(const vector<ustring>& tags_added,
                                     const vector<ustring>& tags_removed)
@@ -174,6 +213,30 @@ namespace paperbox {
             }
         }
     }
+    
+    void
+    DocumentTile::refresh_tag_labels(const vector<ustring>& tags_added,
+                                     const vector<ustring>& tags_removed)
+    {
+        {
+            vector<ustring>::const_iterator it(tags_added.begin());
+            vector<ustring>::const_iterator end(tags_added.end());
+
+            for ( ; it != end; ++it)
+                get_tag_label(it->raw()); // create or do nothing
+        }
+
+        {
+            vector<ustring>::const_iterator it(tags_removed.begin());
+            vector<ustring>::const_iterator end(tags_removed.end());
+
+            for ( ; it != end; ++it) {
+                tag_label_map::iterator label_it = tag_labels_.find(it->raw());
+                if (label_it != tag_labels_.end())
+                    tag_labels_.erase(label_it);
+            }
+        }
+    }
 
     void
     DocumentTile::pack_tag_links()
@@ -184,7 +247,7 @@ namespace paperbox {
         for ( ; it != end; ++it) {
             Gtk::Widget* link_widget = it->second.get();
             link_widget->show();
-            tag_hbox_.pack_start(*link_widget, false, false, 0);
+            tag_links_hbox_.pack_start(*link_widget, false, false, 0);
         }
     }
 
@@ -198,7 +261,32 @@ namespace paperbox {
             TagLinkButton* button =
                 static_cast<TagLinkButton*>(it->second.get());
 
-            tag_hbox_.remove(*button);
+            tag_links_hbox_.remove(*button);
+        }
+    }
+    
+    void
+    DocumentTile::pack_tag_labels()
+    {
+        tag_label_map::iterator it(tag_labels_.begin());
+        tag_label_map::iterator end(tag_labels_.end());
+
+        for ( ; it != end; ++it) {
+            Gtk::Widget* label = it->second.get();
+            label->show();
+            tag_labels_hbox_.pack_start(*label, false, false, 0);
+        }
+    }
+
+    void
+    DocumentTile::unpack_tag_labels()
+    {
+        tag_label_map::iterator it(tag_labels_.begin());
+        tag_label_map::iterator end(tag_labels_.end());
+
+        for ( ; it != end; ++it) {
+            Gtk::Label* label = static_cast<Gtk::Label*>(it->second.get());
+            tag_labels_hbox_.remove(*label);
         }
     }
 
@@ -214,15 +302,18 @@ namespace paperbox {
         Gtk::VBox& content_vbox = get_content_vbox();
 
         if (first_boot_) {
-            content_vbox.pack_start(extra_hbox_, false, false, 0);
-            content_vbox.pack_start(time_hbox_, false, false, 0);
+            content_vbox.pack_start(extra_hbox_, false, false);
+            content_vbox.pack_start(time_hbox_, false, false);
 
-            tag_hbox_.pack_start(tag_tools_hbox_, false, false, 0);
+            tag_hbox_.pack_start(tag_links_hbox_, false, false);
+            tag_hbox_.pack_start(tag_tools_hbox_, false, false);
 
-            content_vbox.show_all();
+            //content_vbox.show_all();
             first_boot_ = false;
         }
 
+        tag_links_hbox_.show_all();
+
         content_vbox.reorder_child(extra_hbox_, 2);
         content_vbox.reorder_child(time_hbox_, 3);
         tag_hbox_.reorder_child(tag_tools_hbox_, 0);
@@ -231,7 +322,10 @@ namespace paperbox {
         time_hbox_.show_all();
         tag_tools_hbox_.show_all();
 
+        tag_labels_hbox_.hide();
+
         get_image().set(thumbnail_large_);
+        restore_title();
     }
 
     void
@@ -241,7 +335,11 @@ namespace paperbox {
         time_hbox_.hide();
         tag_tools_hbox_.hide();
 
+        tag_labels_hbox_.show_all();
+        tag_links_hbox_.hide();
+
         get_image().set(thumbnail_small_);
+        break_title();
     }
 
     void
@@ -267,6 +365,11 @@ namespace paperbox {
         vector<ustring> tags;
         shared_ptr<DialogTagEntry> dialog(DialogTagEntry::create());
         dialog->set_default_response(Gtk::RESPONSE_OK);
+        if (! doc_->get_subject().empty()) {
+            dialog->set_title(doc_->get_subject());
+        } else {
+            dialog->set_title(doc_->get_file_name());
+        }
 
         int response = dialog->run(tags);
 
@@ -277,7 +380,7 @@ namespace paperbox {
     }
 
     void
-    DocumentTile::on_taglink(Gtk::LinkButton */*button*/,
+    DocumentTile::on_taglink(Gtk::LinkButton* /* button */,
                              const ustring& uri)
     {
         signal_tag_clicked_.emit(uri);
@@ -302,6 +405,10 @@ namespace paperbox {
         unpack_tag_links();
         refresh_tag_links(tags_added, tags_removed);
         pack_tag_links();
+
+        unpack_tag_labels();
+        refresh_tag_labels(tags_added, tags_removed);
+        pack_tag_labels();
     }
 
     void
@@ -310,4 +417,43 @@ namespace paperbox {
         on_unselected();
     }
 
+    void
+    DocumentTile::break_title()
+    {
+        Gtk::Label& title_label = get_title_label();
+        title_label.set_ellipsize(Pango::ELLIPSIZE_END);
+        title_label.set_max_width_chars(TITLE_BREAK);
+
+        if (! doc_->get_subject().empty()) {
+            set_title(doc_->get_subject());
+        } else {
+            set_title(doc_->get_file_name());
+        }
+    }
+
+    void
+    DocumentTile::restore_title()
+    {
+        Gtk::Label& title_label = get_title_label();
+        ustring title = get_title();
+
+        title_label.set_max_width_chars(80); // something large enough
+        if (title.size() > TITLE_BREAK) {
+            title.insert(TITLE_BREAK, "\n");
+            set_title(title);
+        }
+    }
+
+    shared_ptr<Document>
+    DocumentTile::get_document()
+    {
+        return doc_;
+    }
+
+    void
+    DocumentTile::show_add_tag_dialog()
+    {
+        on_tag_add_clicked();
+    }
+
 } // namespace paperbox
